#ifndef DefsH
#define DefsH

#include <stdint.h>

//----------------------------------------------------------------------------

//Brightness control modes:
enum BrightMode_t
{
  BRM_OFF, //brightness control off
  BRM_VR,  //brightness control with trimmer pots
  BRM_DAC, //brightness control with built-in DACs
  BRM_PWM  //brightness control with PWM
};

//Display modes:
enum DispMode_t
{
  DP_BAR     = 0x01, //display bars
  DP_PEAK    = 0x02, //display peaks
  DP_BARMAX  = 0x04, //hold bars max
  DP_PEAKMAX = 0x08, //hold peaks max
  DP_ABSMAX  = 0x10  //display statistics
};

//----------------------------------------------------------------------------

//parameters set:
typedef struct 
{
  uint16_t integ;
  uint16_t decay;
  uint16_t scale;
  uint16_t resp;
  uint16_t fall;
  uint16_t hold;
  uint16_t mfall;
  uint16_t mhold;
} params_t;

//----------------------------------------------------------------------------

static const double INTEG_TTAU = 4.0; //T_int/tau_int ratio for -2 dB (Sin)
static const double DECAY_TTAU = 2.3; //T_dec/tau_dec ratio for -20 dB (DC)
static const double RESP_TTAU  = 2.3; //T_resp/tau_resp ratio for -1 dB (DC)
static const double FALL_TTAU  = 2.3; //T_fall/tau_fall ratio for -20 dB (DC)

static const int16_t DATA_BUFF = 8; //result buffer size

//----------------------------------------------------------------------------

//0.01 dB step table (0.00dB to -0.15dB):
static const uint16_t DB_001[16] =
{ 65535, 65460, 65384, 65309, 65234, 65159, 65084, 65009,
  64934, 64859, 64785, 64710, 64636, 64561, 64487, 64413 };

//0.16 dB step table (0.00dB to -2.40dB):
static const uint16_t DB_016[16] =
{ 65535, 64339, 63165, 62012, 60880, 59769, 58678, 57607,
  56555, 55523, 54510, 53515, 52538, 51579, 50638, 49713 };

//2.56 dB step table (0.00dB to -38.40dB):
static const uint16_t DB_256[16] =
{ 65535, 48806, 36347, 27069, 20159, 15013, 11181, 8327,
   6201,  4618,  3439,  2561,  1908,  1421,  1058,  788 };

//40.96dB step table (0.00dB to -40.96dB):
static const uint16_t DB_4096[2] =
{ 65535, 587 };

uint16_t __fastcall dB2Code(uint16_t dB)
{
  if(dB > 8000) return(0);
  uint32_t y = DB_001[dB & 0xF];
  y = y *  DB_016[(dB >> 4)  & 0xF] / 65536;
  y = y *  DB_256[(dB >> 8)  & 0xF] / 65536;
  y = y * DB_4096[(dB >> 12) & 0x1] / 65536;
  y = y * 10000 / 65536;
  return(y);
}

//----------------------------------------------------------------------------

//bars segments count:
static const uint8_t SEGS = 50;

//LEDs masks:
enum {
  TRNON = 0x0000,
  TR0   = 0x0001,
  TR1   = 0x0002,
  TR2   = 0x0004,
  TR3   = 0x0008,
  TR4   = 0x0010,
  TR5   = 0x0020,
  TR6   = 0x0040,
  TR7   = 0x0080,
  TR8   = 0x0100,
  TR9   = 0x0200,
  SCALE = 0x0400,
  TRALL = 0x03FF,
  TSALL = 0x07FF
};

//----------------------------------------------------------------------------

#endif
